@@ -92,7 +92,6 @@ typedef enum {
 	BVW_ERROR_CANNOT_CAPTURE,
 	BVW_ERROR_READ_ERROR,
 	BVW_ERROR_PLUGIN_LOAD,
-	BVW_ERROR_STILL_IMAGE,
 	BVW_ERROR_EMPTY_FILE
 } BvwError;
 
@@ -200,6 +199,7 @@ typedef enum {
 	BVW_INFO_VIDEO_BITRATE,
 	BVW_INFO_VIDEO_CODEC,
 	BVW_INFO_FPS,
+	BVW_INFO_STILL_IMAGE,
 	/* Audio */
 	BVW_INFO_HAS_AUDIO,
 	BVW_INFO_AUDIO_BITRATE,
