@@ -50,11 +50,11 @@ enum
 struct _EvSidebarPrivate {
 	GtkWidget *notebook;
 	GtkWidget *menu;
-	GtkWidget *frame;
 	GtkWidget *hbox;
 	GtkWidget *label;
-	   
+
 	GtkTreeModel *page_model;
+	char *current;
 };
 
 enum {
@@ -74,11 +74,14 @@ ev_sidebar_destroy (GtkObject *object)
 {
 	EvSidebar *ev_sidebar = EV_SIDEBAR (object);
 
+	g_free (ev_sidebar->priv->current);
+	ev_sidebar->priv->current = NULL;
+
 	if (ev_sidebar->priv->menu) {
 		gtk_menu_detach (GTK_MENU (ev_sidebar->priv->menu));
 		ev_sidebar->priv->menu = NULL;
 	}
-	   
+
 	(* GTK_OBJECT_CLASS (ev_sidebar_parent_class)->destroy) (object);
 }
 
@@ -138,6 +141,14 @@ ev_sidebar_select_button_press_cb (GtkWidget      *widget,
 	EvSidebar *ev_sidebar = EV_SIDEBAR (user_data);
 
 	if (event->button == 1) {
+		GtkRequisition requisition;
+		gint width;
+
+		width = widget->allocation.width;
+		gtk_widget_set_size_request (ev_sidebar->priv->menu, -1, -1);
+		gtk_widget_size_request (ev_sidebar->priv->menu, &requisition);
+		gtk_widget_set_size_request (ev_sidebar->priv->menu,
+				MAX (width, requisition.width), -1);
 		gtk_widget_grab_focus (widget);
 			 
 		gtk_toggle_button_set_active (GTK_TOGGLE_BUTTON (widget), TRUE);
@@ -210,16 +221,17 @@ ev_sidebar_menu_item_activate_cb (GtkWidget *widget,
 	EvSidebar *ev_sidebar = EV_SIDEBAR (user_data);
 	GtkTreeIter iter;
 	GtkWidget *menu_item, *item;
-	gchar *title;
+	gchar *title, *page_id;
 	gboolean valid;
 	gint index;
 
 	menu_item = gtk_menu_get_active (GTK_MENU (ev_sidebar->priv->menu));
 	valid = gtk_tree_model_get_iter_first (ev_sidebar->priv->page_model, &iter);
-	   
+
 	while (valid) {
 		gtk_tree_model_get (ev_sidebar->priv->page_model,
 				    &iter,
+				    PAGE_COLUMN_ID, &page_id,
 				    PAGE_COLUMN_TITLE, &title, 
 				    PAGE_COLUMN_MENU_ITEM, &item,
 				    PAGE_COLUMN_NOTEBOOK_INDEX, &index,
@@ -229,17 +241,21 @@ ev_sidebar_menu_item_activate_cb (GtkWidget *widget,
 			gtk_notebook_set_current_page
 				(GTK_NOTEBOOK (ev_sidebar->priv->notebook), index);
 			gtk_label_set_text (GTK_LABEL (ev_sidebar->priv->label), title);
+			g_free (ev_sidebar->priv->current);
+			ev_sidebar->priv->current = page_id;
 			valid = FALSE;
 		} else {
 			valid = gtk_tree_model_iter_next (ev_sidebar->priv->page_model, &iter);
+			g_free (page_id);
 		}
+		g_object_unref (item);
+		g_free (title);
 	}
 }
 
 static void
 ev_sidebar_init (EvSidebar *ev_sidebar)
 {
-	GtkWidget *frame;
 	GtkWidget *hbox;
 	GtkWidget *close_button;
 	GtkWidget *select_button;
@@ -259,14 +275,9 @@ ev_sidebar_init (EvSidebar *ev_sidebar)
 					    G_TYPE_INT);
 
 	/* top option menu */
-	frame = gtk_frame_new (NULL);
-	ev_sidebar->priv->frame = frame;
-	gtk_box_pack_start (GTK_BOX (ev_sidebar), frame, FALSE, FALSE, 0);
-	gtk_widget_show (frame);
-
 	hbox = gtk_hbox_new (FALSE, 0);
 	ev_sidebar->priv->hbox = hbox;
-	gtk_container_add (GTK_CONTAINER (frame), hbox);
+	gtk_box_pack_start (GTK_BOX (ev_sidebar), hbox, FALSE, FALSE, 0);
 	gtk_widget_show (hbox);
 
 	select_button = gtk_toggle_button_new ();
@@ -303,7 +314,7 @@ ev_sidebar_init (EvSidebar *ev_sidebar)
 			  ev_sidebar);
 	   
 	image = gtk_image_new_from_stock (GTK_STOCK_CLOSE,
-					  GTK_ICON_SIZE_SMALL_TOOLBAR);
+					  GTK_ICON_SIZE_MENU);
 	gtk_container_add (GTK_CONTAINER (close_button), image);
 	gtk_widget_show (image);
    
@@ -323,7 +334,7 @@ ev_sidebar_init (EvSidebar *ev_sidebar)
 	gtk_notebook_set_show_border (GTK_NOTEBOOK (ev_sidebar->priv->notebook), FALSE);
 	gtk_notebook_set_show_tabs (GTK_NOTEBOOK (ev_sidebar->priv->notebook), FALSE);
 	gtk_box_pack_start (GTK_BOX (ev_sidebar), ev_sidebar->priv->notebook,
-			    TRUE, TRUE, 0);
+			    TRUE, TRUE, 6);
 	gtk_widget_show (ev_sidebar->priv->notebook);
 }
 
@@ -339,6 +350,52 @@ ev_sidebar_new (void)
 	return ev_sidebar;
 }
 
+const char *
+ev_sidebar_get_current_page (EvSidebar *ev_sidebar)
+{
+	g_return_val_if_fail (EV_IS_SIDEBAR (ev_sidebar), NULL);
+	g_return_val_if_fail (ev_sidebar->priv != NULL, NULL);
+
+	return ev_sidebar->priv->current;
+}
+
+void
+ev_sidebar_set_current_page (EvSidebar *ev_sidebar, const char *new_page_id)
+{
+	GtkTreeIter iter;
+	GtkWidget *menu_item;
+	gchar *page_id;
+	gboolean valid;
+	gint index;
+
+	g_return_if_fail (EV_IS_SIDEBAR (ev_sidebar));
+	g_return_if_fail (new_page_id != NULL);
+
+	if (strcmp (new_page_id, ev_sidebar->priv->current) == 0)
+		return;
+
+	valid = gtk_tree_model_get_iter_first (ev_sidebar->priv->page_model, &iter);
+
+	while (valid) {
+		gtk_tree_model_get (ev_sidebar->priv->page_model,
+				    &iter,
+				    PAGE_COLUMN_ID, &page_id,
+				    PAGE_COLUMN_MENU_ITEM, &menu_item,
+				    PAGE_COLUMN_NOTEBOOK_INDEX, &index,
+				    -1);
+
+		if (page_id != NULL && strcmp (new_page_id, page_id) == 0) {
+			gtk_menu_set_active (GTK_MENU (ev_sidebar->priv->menu), index);
+			gtk_menu_item_activate (GTK_MENU_ITEM (menu_item));
+			valid = FALSE;
+		} else {
+			valid = gtk_tree_model_iter_next (ev_sidebar->priv->page_model, &iter);
+		}
+		g_object_unref (menu_item);
+		g_free (page_id);
+	}
+}
+
 void
 ev_sidebar_add_page (EvSidebar   *ev_sidebar,
 		     const gchar *page_id,
@@ -347,7 +404,7 @@ ev_sidebar_add_page (EvSidebar   *ev_sidebar,
 {
 	GtkTreeIter iter;
 	GtkWidget *menu_item;
-	gchar *label_title;
+	gchar *label_title, *new_page_id;
 	int index;
 	   
 	g_return_if_fail (EV_IS_SIDEBAR (ev_sidebar));
@@ -365,7 +422,7 @@ ev_sidebar_add_page (EvSidebar   *ev_sidebar,
 	gtk_widget_show (menu_item);
 	gtk_menu_shell_append (GTK_MENU_SHELL (ev_sidebar->priv->menu),
 			       menu_item);
-	   
+
 	gtk_list_store_insert_with_values (GTK_LIST_STORE (ev_sidebar->priv->page_model),
 					   &iter, 0,
 					   PAGE_COLUMN_ID, page_id,
@@ -379,6 +436,7 @@ ev_sidebar_add_page (EvSidebar   *ev_sidebar,
 	gtk_tree_model_get_iter_first (ev_sidebar->priv->page_model, &iter);
 	gtk_tree_model_get (ev_sidebar->priv->page_model,
 			    &iter,
+			    PAGE_COLUMN_ID, &new_page_id,
 			    PAGE_COLUMN_TITLE, &label_title,
 			    PAGE_COLUMN_NOTEBOOK_INDEX, &index,
 			    -1);
@@ -387,5 +445,7 @@ ev_sidebar_add_page (EvSidebar   *ev_sidebar,
 	gtk_label_set_text (GTK_LABEL (ev_sidebar->priv->label), label_title);
 	gtk_notebook_set_current_page (GTK_NOTEBOOK (ev_sidebar->priv->notebook),
 				       index);
+	ev_sidebar->priv->current = new_page_id;
+	g_free (label_title);
 }
 
