@@ -10,6 +10,7 @@ static gboolean option_recurse = TRUE;
 static gboolean option_debug = FALSE;
 static gboolean option_data = FALSE;
 static gboolean option_force = FALSE;
+static gboolean option_disable_unsafe = FALSE;
 
 static void
 header (const char *message)
@@ -226,6 +227,7 @@ test_parsing (void)
 	g_object_set (G_OBJECT (pl), "recurse", option_recurse, NULL);
 	g_object_set (G_OBJECT (pl), "debug", option_debug, NULL);
 	g_object_set (G_OBJECT (pl), "force", option_force, NULL);
+	g_object_set (G_OBJECT (pl), "disable-unsafe", option_disable_unsafe, NULL);
 	g_signal_connect (G_OBJECT (pl), "entry", G_CALLBACK (entry_added), NULL);
 	g_signal_connect (G_OBJECT (pl), "playlist-start", G_CALLBACK (playlist_started), NULL);
 	g_signal_connect (G_OBJECT (pl), "playlist-end", G_CALLBACK (playlist_ended), NULL);
@@ -252,7 +254,7 @@ int main (int argc, char **argv)
 			argv++;
 			argc--;
 		} else if (strcmp (argv[1], "--help") == 0 || strcmp (argv[1], "-h") == 0) {
-			g_print ("Usage: %s <-n | --no-recurse> <-d | --debug> <-h | --help> <-t | --data > <url>\n", argv[0]);
+			g_print ("Usage: %s <-n | --no-recurse> <-d | --debug> <-h | --help> <-t | --data > <-u | --disable-unsafe> <url>\n", argv[0]);
 			return 0;
 		} else if (strcmp (argv[1], "--data") == 0 || strcmp (argv[1], "-t") == 0) {
 			g_print ("Using data, instead of filenames\n");
@@ -264,6 +266,11 @@ int main (int argc, char **argv)
 			option_force = TRUE;
 			argv++;
 			argc--;
+		} else if (strcmp (argv[1], "--disable-unsafe") == 0 || strcmp (argv[1], "-u") == 0) {
+			g_print ("Disabling unsafe playlist-types\n");
+			option_disable_unsafe = TRUE;
+			argv++;
+			argc--;
 		} else /* other options here */ {
 			break;
 		}
