@@ -48,6 +48,7 @@
 #define RECURSE_LEVEL_MAX 4
 #define DIR_MIME_TYPE "x-directory/normal"
 #define BLOCK_DEVICE_TYPE "x-special/device-block"
+#define EMPTY_FILE_TYPE "application/x-zerosize"
 #define EXTINF "#EXTINF:"
 #define DEBUG(x) { if (parser->priv->debug) x; }
 
@@ -63,6 +64,9 @@ typedef struct {
 	PlaylistCallback func;
 #endif
 	PlaylistIdenCallback iden;
+#ifndef TOTEM_PL_PARSER_MINI
+	guint unsafe : 1;
+#endif
 } PlaylistTypes;
 
 static gboolean totem_pl_parser_is_ra (const char *data, gsize len);
@@ -85,6 +89,7 @@ struct TotemPlParserPrivate
 	guint recurse : 1;
 	guint debug : 1;
 	guint force : 1;
+	guint disable_unsafe : 1;
 };
 
 static void totem_pl_parser_set_property (GObject *object,
@@ -100,7 +105,8 @@ enum {
 	PROP_NONE,
 	PROP_RECURSE,
 	PROP_DEBUG,
-	PROP_FORCE
+	PROP_FORCE,
+	PROP_DISABLE_UNSAFE
 };
 
 /* Signals */
@@ -154,6 +160,14 @@ totem_pl_parser_class_init (TotemPlParserClass *klass)
 							       FALSE,
 							       G_PARAM_READWRITE));
 
+	g_object_class_install_property (object_class,
+					 PROP_DISABLE_UNSAFE,
+					 g_param_spec_boolean ("disable-unsafe",
+							       "disable-unsafe",
+							       "Whether or not to disable parsing of unsafe locations", 
+							       FALSE,
+							       G_PARAM_READWRITE));
+
 	/* Signals */
 	totem_pl_parser_table_signals[ENTRY] =
 		g_signal_new ("entry",
@@ -200,6 +214,9 @@ totem_pl_parser_set_property (GObject *object,
 	case PROP_FORCE:
 		parser->priv->force = g_value_get_boolean (value) != FALSE;
 		break;
+	case PROP_DISABLE_UNSAFE:
+		parser->priv->disable_unsafe = g_value_get_boolean (value) != FALSE;
+		break;
 	default:
 		G_OBJECT_WARN_INVALID_PROPERTY_ID (object, prop_id, pspec);
 		break;
@@ -225,6 +242,9 @@ totem_pl_parser_get_property (GObject *object,
 	case PROP_FORCE:
 		g_value_set_boolean (value, parser->priv->force);
 		break;
+	case PROP_DISABLE_UNSAFE:
+		g_value_set_boolean (value, parser->priv->disable_unsafe);
+		break;
 	default:
 		G_OBJECT_WARN_INVALID_PROPERTY_ID (object, prop_id, pspec);
 		break;
@@ -259,17 +279,18 @@ totem_pl_parser_new (void)
 	return TOTEM_PL_PARSER (g_object_new (TOTEM_TYPE_PL_PARSER, NULL));
 }
 
-static const char *
+static char *
 my_gnome_vfs_get_mime_type_with_data (const char *uri, gpointer *data, TotemPlParser *parser)
 {
 	GnomeVFSResult result;
 	GnomeVFSHandle *handle;
-	char *buffer;
+	char *buffer, *server_mimetype;
 	const char *mimetype;
 	GnomeVFSFileSize total_bytes_read;
 	GnomeVFSFileSize bytes_read;
 
 	*data = NULL;
+	server_mimetype = NULL;
 
 	/* Stat for a block device, we're screwed as far as speed
 	 * is concerned now */
@@ -277,7 +298,7 @@ my_gnome_vfs_get_mime_type_with_data (const char *uri, gpointer *data, TotemPlPa
 		struct stat buf;
 		if (stat (uri + strlen ("file://"), &buf) == 0) {
 			if (S_ISBLK (buf.st_mode))
-				return BLOCK_DEVICE_TYPE;
+				return g_strdup (BLOCK_DEVICE_TYPE);
 		}
 	}
 
@@ -285,12 +306,25 @@ my_gnome_vfs_get_mime_type_with_data (const char *uri, gpointer *data, TotemPlPa
 	result = gnome_vfs_open (&handle, uri, GNOME_VFS_OPEN_READ);
 	if (result != GNOME_VFS_OK) {
 		if (result == GNOME_VFS_ERROR_IS_DIRECTORY)
-			return DIR_MIME_TYPE;
+			return g_strdup (DIR_MIME_TYPE);
 		DEBUG(g_print ("URL '%s' couldn't be opened in _get_mime_type_with_data: '%s'\n", uri, gnome_vfs_result_to_string (result)));
 		return NULL;
 	}
 	DEBUG(g_print ("URL '%s' was opened successfully in _get_mime_type_with_data:\n", uri));
 
+	/* Try to get a mime-type, from the server itself */
+	if (g_str_has_prefix (uri, "http://") != FALSE) {
+		GnomeVFSResult res;
+		GnomeVFSFileInfo *info;
+		info = gnome_vfs_file_info_new ();
+		res = gnome_vfs_get_file_info_from_handle (handle,
+				info, GNOME_VFS_FILE_INFO_GET_MIME_TYPE);
+		if (res == GNOME_VFS_OK)
+			server_mimetype = g_strdup (info->mime_type);
+		gnome_vfs_file_info_unref (info);
+	}
+
+
 	/* Read the whole thing, up to MIME_READ_CHUNK_SIZE */
 	buffer = NULL;
 	total_bytes_read = 0;
@@ -330,6 +364,12 @@ my_gnome_vfs_get_mime_type_with_data (const char *uri, gpointer *data, TotemPlPa
 		return NULL;
 	}
 
+	/* Empty file */
+	if (total_bytes_read == 0) {
+		DEBUG(g_print ("URL '%s' is empty in _get_mime_type_with_data\n", uri));
+		return g_strdup (EMPTY_FILE_TYPE);
+	}
+
 	/* Return the file null-terminated. */
 	buffer = g_realloc (buffer, total_bytes_read + 1);
 	buffer[total_bytes_read] = '\0';
@@ -337,7 +377,12 @@ my_gnome_vfs_get_mime_type_with_data (const char *uri, gpointer *data, TotemPlPa
 
 	mimetype = gnome_vfs_get_mime_type_for_data (*data, total_bytes_read);
 
-	return mimetype;
+	if (server_mimetype != NULL && strcmp (mimetype, "text/plain") == 0) {
+		return server_mimetype;
+	}
+	g_free (server_mimetype);
+
+	return g_strdup (mimetype);
 }
 
 static char *
@@ -354,6 +399,39 @@ totem_pl_parser_read_entire_file (const char *url, int *size)
 	return contents;
 }
 
+static xmlDocPtr
+totem_pl_parser_parse_xml_file (const char *url)
+{
+	xmlDocPtr doc;
+	char *contents;
+	int size;
+
+	contents = totem_pl_parser_read_entire_file (url, &size);
+	if (contents == NULL)
+		return NULL;
+
+	/* Try to remove HTML style comments */
+	{
+		char *needle;
+
+		while ((needle = strstr (contents, "<!--")) != NULL) {
+			while (strncmp (needle, "-->", 3) != 0) {
+				*needle = ' ';
+				needle++;
+				if (*needle == '\0')
+					break;
+			}
+		}
+	}
+
+	doc = xmlParseMemory (contents, size);
+	if (doc == NULL)
+		doc = xmlRecoverMemory (contents, size);
+	g_free (contents);
+
+	return doc;
+}
+
 static char*
 totem_pl_parser_base_url (const char *url)
 {
@@ -379,6 +457,21 @@ totem_pl_parser_base_url (const char *url)
 	return base;
 }
 
+static gboolean
+totem_pl_parser_line_is_empty (const char *line)
+{
+	guint i;
+
+	if (line == NULL)
+		return TRUE;
+
+	for (i = 0; line[i] != '\0'; i++) {
+		if (line[i] != '\t' && line[i] != ' ')
+			return FALSE;
+	}
+	return TRUE;
+}
+
 static gboolean
 write_string (GnomeVFSHandle *handle, const char *buf, GError **error)
 {
@@ -1008,7 +1101,8 @@ totem_pl_parser_add_ram (TotemPlParser *parser, const char *url, gpointer data)
 	g_free (contents);
 
 	for (i = 0; lines[i] != NULL; i++) {
-		if (strcmp (lines[i], "") == 0)
+		/* Empty line */
+		if (totem_pl_parser_line_is_empty (lines[i]) != FALSE)
 			continue;
 
 		retval = TOTEM_PL_PARSER_RESULT_SUCCESS;
@@ -1092,7 +1186,7 @@ static TotemPlParserResult
 totem_pl_parser_add_asf_parser (TotemPlParser *parser,
 		const char *url, gpointer data)
 {
-	gboolean retval = TOTEM_PL_PARSER_RESULT_UNHANDLED;
+	TotemPlParserResult retval = TOTEM_PL_PARSER_RESULT_UNHANDLED;
 	char *contents, *ref;
 	int size;
 
@@ -1124,7 +1218,7 @@ totem_pl_parser_add_asf_parser (TotemPlParser *parser,
 static TotemPlParserResult
 totem_pl_parser_add_pls_with_contents (TotemPlParser *parser, const char *url, const char *contents)
 {
-	gboolean retval = TOTEM_PL_PARSER_RESULT_UNHANDLED;
+	TotemPlParserResult retval = TOTEM_PL_PARSER_RESULT_UNHANDLED;
 	char **lines;
 	int i, num_entries;
 	char *split_char, *playlist_title;
@@ -1145,7 +1239,7 @@ totem_pl_parser_add_pls_with_contents (TotemPlParser *parser, const char *url, c
 	playlist_title = NULL;
 
 	/* Ignore empty lines */
-	while (lines[i] != NULL && strcmp (lines[i], "") == 0)
+	while (totem_pl_parser_line_is_empty (lines[i]) != FALSE)
 		i++;
 
 	if (lines[i] == NULL
@@ -1246,7 +1340,7 @@ bail:
 static TotemPlParserResult
 totem_pl_parser_add_pls (TotemPlParser *parser, const char *url, gpointer data)
 {
-	gboolean retval = TOTEM_PL_PARSER_RESULT_UNHANDLED;
+	TotemPlParserResult retval = TOTEM_PL_PARSER_RESULT_UNHANDLED;
 	char *contents;
 	int size;
 
@@ -1503,18 +1597,10 @@ totem_pl_parser_add_asx (TotemPlParser *parser, const char *url, gpointer data)
 {
 	xmlDocPtr doc;
 	xmlNodePtr node;
-	char *contents = NULL, *base;
-	int size;
-	gboolean retval = TOTEM_PL_PARSER_RESULT_UNHANDLED;
+	char *base;
+	TotemPlParserResult retval = TOTEM_PL_PARSER_RESULT_UNHANDLED;
 
-	contents = totem_pl_parser_read_entire_file (url, &size);
-	if (contents == NULL)
-		return FALSE;
-
-	doc = xmlParseMemory (contents, size);
-	if (doc == NULL)
-		doc = xmlRecoverMemory (contents, size);
-	g_free (contents);
+	doc = totem_pl_parser_parse_xml_file (url);
 
 	/* If the document has no root, or no name */
 	if(!doc || !doc->children || !doc->children->name) {
@@ -1679,18 +1765,10 @@ totem_pl_parser_add_smil (TotemPlParser *parser, const char *url, gpointer data)
 {
 	xmlDocPtr doc;
 	xmlNodePtr node;
-	char *contents = NULL, *base;
-	int size;
+	char *base;
 	TotemPlParserResult retval = TOTEM_PL_PARSER_RESULT_UNHANDLED;
 
-	contents = totem_pl_parser_read_entire_file (url, &size);
-	if (contents == NULL)
-		return TOTEM_PL_PARSER_RESULT_ERROR;
-
-	doc = xmlParseMemory (contents, size);
-	if (doc == NULL)
-		doc = xmlRecoverMemory (contents, size);
-	g_free (contents);
+	doc = totem_pl_parser_parse_xml_file (url);
 
 	/* If the document has no root, or no name */
 	if(!doc || !doc->children
@@ -1731,23 +1809,40 @@ totem_pl_parser_add_asf (TotemPlParser *parser, const char *url, gpointer data)
 }
 
 static TotemPlParserResult
-totem_pl_parser_add_quicktime_metalink (TotemPlParser *parser, const char *url, gpointer data)
+totem_pl_parser_add_quicktime_rtsptextrtsp (TotemPlParser *parser, const char *url, gpointer data)
 {
-	xmlDocPtr doc;
-	xmlNodePtr node;
 	char *contents = NULL;
-	xmlChar *src;
 	int size;
+	char **lines;
 
 	contents = totem_pl_parser_read_entire_file (url, &size);
 	if (contents == NULL)
 		return TOTEM_PL_PARSER_RESULT_ERROR;
 
-	doc = xmlParseMemory (contents, size);
-	if (doc == NULL)
-		doc = xmlRecoverMemory (contents, size);
+	lines = g_strsplit (contents, "\n", 0);
 	g_free (contents);
 
+	totem_pl_parser_add_one_url (parser, lines[0] + strlen ("RTSPtext"), NULL);
+	g_strfreev (lines);
+
+	return TOTEM_PL_PARSER_RESULT_SUCCESS;
+}
+
+static TotemPlParserResult
+totem_pl_parser_add_quicktime_metalink (TotemPlParser *parser, const char *url, gpointer data)
+{
+	xmlDocPtr doc;
+	xmlNodePtr node;
+	xmlChar *src;
+
+	if (g_str_has_prefix (data, "RTSPtextRTSP://") != FALSE
+			|| g_str_has_prefix (data, "rtsptextrtsp://") != FALSE
+			|| g_str_has_prefix (data, "RTSPtextrtsp://") != FALSE) {
+		return totem_pl_parser_add_quicktime_rtsptextrtsp (parser, url, data);
+	}
+
+	doc = totem_pl_parser_parse_xml_file (url);
+
 	/* If the document has no root, or no name */
 	if(!doc || !doc->children
 			|| !doc->children->name
@@ -2050,18 +2145,10 @@ totem_pl_parser_add_xspf (TotemPlParser *parser, const char *url,
 {
 	xmlDocPtr doc;
 	xmlNodePtr node;
-	char *contents = NULL, *base;
-	int size;
-	gboolean retval = TOTEM_PL_PARSER_RESULT_UNHANDLED;
+	char *base;
+	TotemPlParserResult retval = TOTEM_PL_PARSER_RESULT_UNHANDLED;
 
-	contents = totem_pl_parser_read_entire_file (url, &size);
-	if (contents == NULL)
-		return FALSE;
-
-	doc = xmlParseMemory (contents, size);
-	if (doc == NULL)
-		doc = xmlRecoverMemory (contents, size);
-	g_free (contents);
+	doc = totem_pl_parser_parse_xml_file (url);
 
 	/* If the document has no root, or no name */
 	if(!doc || !doc->children
@@ -2151,13 +2238,23 @@ totem_pl_parser_add_directory (TotemPlParser *parser, const char *url,
 static gboolean
 totem_pl_parser_is_ra (const char *data, gsize len)
 {
+	guint i;
+	const char *buf;
+
 	/* pnm is the smallest, 6 chars */
 	if (len < 7)
 		return FALSE;
 
-	if (g_str_has_prefix (data, "http://") != FALSE
-			|| g_str_has_prefix (data, "rtsp://") != FALSE
-			|| g_str_has_prefix (data, "pnm://") != FALSE) {
+	/* Start ignoring the spaces and such */
+	for (i = 0; i < len; i++) {
+		if (data[i] != '\n' && data[i] != '\t' && data[i] != ' ')
+			break;
+	}
+	buf = data + i;
+
+	if (g_str_has_prefix (buf, "http://") != FALSE
+			|| g_str_has_prefix (buf, "rtsp://") != FALSE
+			|| g_str_has_prefix (buf, "pnm://") != FALSE) {
 		return TRUE;
 	}
 	return FALSE;
@@ -2206,6 +2303,15 @@ totem_pl_parser_is_quicktime (const char *data, gsize len)
 	if (len > MIME_READ_CHUNK_SIZE)
 		len = MIME_READ_CHUNK_SIZE;
 
+	/* Check for RTSPtextRTSP Quicktime references */
+	if (len <= strlen ("RTSPtextRTSP://"))
+		return FALSE;
+	if (g_str_has_prefix (data, "RTSPtextRTSP://") != FALSE
+			|| g_str_has_prefix (data, "rtsptextrtsp://") != FALSE
+			|| g_str_has_prefix (data, "RTSPtextrtsp://") != FALSE) {
+		return TRUE;
+	}
+
 	/* FIXME would be nicer to have an strnstr */
 	buffer = g_memdup (data, len);
 	if (buffer == NULL) {
@@ -2222,31 +2328,33 @@ totem_pl_parser_is_quicktime (const char *data, gsize len)
 }
 
 #ifndef TOTEM_PL_PARSER_MINI
-#define PLAYLIST_TYPE(mime,cb,identcb) { mime, cb, identcb }
+#define PLAYLIST_TYPE(mime,cb,identcb,unsafe) { mime, cb, identcb, unsafe }
 #define PLAYLIST_TYPE2(mime,cb,identcb) { mime, cb, identcb }
+#define PLAYLIST_TYPE3(mime) { mime, NULL, NULL, FALSE }
 #else
-#define PLAYLIST_TYPE(mime,cb,identcb) { mime }
+#define PLAYLIST_TYPE(mime,cb,identcb,unsafe) { mime }
 #define PLAYLIST_TYPE2(mime,cb,identcb) { mime, identcb }
+#define PLAYLIST_TYPE3(mime) { mime }
 #endif
 
 /* These ones need a special treatment, mostly parser formats */
 static PlaylistTypes special_types[] = {
-	PLAYLIST_TYPE ("audio/x-mpegurl", totem_pl_parser_add_m3u, NULL),
-	PLAYLIST_TYPE ("audio/playlist", totem_pl_parser_add_m3u, NULL),
-	PLAYLIST_TYPE ("audio/x-ms-asx", totem_pl_parser_add_asx, NULL),
-	PLAYLIST_TYPE ("audio/x-scpls", totem_pl_parser_add_pls, NULL),
-	PLAYLIST_TYPE ("application/x-smil", totem_pl_parser_add_smil, NULL),
-	PLAYLIST_TYPE ("application/smil", totem_pl_parser_add_smil, NULL),
-	PLAYLIST_TYPE ("video/x-ms-wvx", totem_pl_parser_add_asx, NULL),
-	PLAYLIST_TYPE ("audio/x-ms-wax", totem_pl_parser_add_asx, NULL),
-	PLAYLIST_TYPE ("application/xspf+xml", totem_pl_parser_add_xspf, NULL),
+	PLAYLIST_TYPE ("audio/x-mpegurl", totem_pl_parser_add_m3u, NULL, FALSE),
+	PLAYLIST_TYPE ("audio/playlist", totem_pl_parser_add_m3u, NULL, FALSE),
+	PLAYLIST_TYPE ("audio/x-ms-asx", totem_pl_parser_add_asx, NULL, FALSE),
+	PLAYLIST_TYPE ("audio/x-scpls", totem_pl_parser_add_pls, NULL, FALSE),
+	PLAYLIST_TYPE ("application/x-smil", totem_pl_parser_add_smil, NULL, FALSE),
+	PLAYLIST_TYPE ("application/smil", totem_pl_parser_add_smil, NULL, FALSE),
+	PLAYLIST_TYPE ("video/x-ms-wvx", totem_pl_parser_add_asx, NULL, FALSE),
+	PLAYLIST_TYPE ("audio/x-ms-wax", totem_pl_parser_add_asx, NULL, FALSE),
+	PLAYLIST_TYPE ("application/xspf+xml", totem_pl_parser_add_xspf, NULL, FALSE),
 #ifndef TOTEM_PL_PARSER_MINI
-	PLAYLIST_TYPE ("application/x-desktop", totem_pl_parser_add_desktop, NULL),
-	PLAYLIST_TYPE ("application/x-gnome-app-info", totem_pl_parser_add_desktop, NULL),
-	PLAYLIST_TYPE ("application/x-cd-image", totem_pl_parser_add_iso, NULL),
-	PLAYLIST_TYPE ("application/x-cue", totem_pl_parser_add_cue, NULL),
-	PLAYLIST_TYPE ("x-directory/normal", totem_pl_parser_add_directory, NULL),
-	PLAYLIST_TYPE ("x-special/device-block", totem_pl_parser_add_block, NULL),
+	PLAYLIST_TYPE ("application/x-desktop", totem_pl_parser_add_desktop, NULL, TRUE),
+	PLAYLIST_TYPE ("application/x-gnome-app-info", totem_pl_parser_add_desktop, NULL, TRUE),
+	PLAYLIST_TYPE ("application/x-cd-image", totem_pl_parser_add_iso, NULL, TRUE),
+	PLAYLIST_TYPE ("application/x-cue", totem_pl_parser_add_cue, NULL, TRUE),
+	PLAYLIST_TYPE ("x-directory/normal", totem_pl_parser_add_directory, NULL, TRUE),
+	PLAYLIST_TYPE ("x-special/device-block", totem_pl_parser_add_block, NULL, TRUE),
 #endif
 };
 
@@ -2263,16 +2371,17 @@ static PlaylistTypes dual_types[] = {
 	PLAYLIST_TYPE2 ("video/x-ms-wmv", totem_pl_parser_add_asf, totem_pl_parser_is_asf),
 	PLAYLIST_TYPE2 ("video/quicktime", totem_pl_parser_add_quicktime, totem_pl_parser_is_quicktime),
 	PLAYLIST_TYPE2 ("application/x-quicktime-media-link", totem_pl_parser_add_quicktime, totem_pl_parser_is_quicktime),
+	PLAYLIST_TYPE2 ("application/x-quicktimeplayer", totem_pl_parser_add_quicktime, totem_pl_parser_is_quicktime),
 };
 
 #ifndef TOTEM_PL_PARSER_MINI
 
 static PlaylistTypes ignore_types[] = {
-	PLAYLIST_TYPE ("image/*", NULL, NULL),
-	PLAYLIST_TYPE ("text/plain", NULL, NULL),
-	PLAYLIST_TYPE ("application/x-rar", NULL, NULL),
-	PLAYLIST_TYPE ("application/zip", NULL, NULL),
-	PLAYLIST_TYPE ("application/x-trash", NULL, NULL),
+	PLAYLIST_TYPE3 ("image/*"),
+	PLAYLIST_TYPE3 ("text/plain"),
+	PLAYLIST_TYPE3 ("application/x-rar"),
+	PLAYLIST_TYPE3 ("application/zip"),
+	PLAYLIST_TYPE3 ("application/x-trash"),
 };
 
 static gboolean
@@ -2375,7 +2484,7 @@ totem_pl_parser_ignore_from_mimetype (TotemPlParser *parser, const char *mimetyp
 static TotemPlParserResult
 totem_pl_parser_parse_internal (TotemPlParser *parser, const char *url)
 {
-	const char *mimetype;
+	char *mimetype;
 	guint i;
 	gpointer data = NULL;
 	TotemPlParserResult ret = TOTEM_PL_PARSER_RESULT_ERROR;
@@ -2400,10 +2509,10 @@ totem_pl_parser_parse_internal (TotemPlParser *parser, const char *url)
 	if (parser->priv->force != FALSE) {
 		mimetype = my_gnome_vfs_get_mime_type_with_data (url, &data, parser);
 	} else {
-		mimetype = gnome_vfs_mime_type_from_name (url);
+		mimetype = g_strdup (gnome_vfs_get_mime_type_for_name (url));
 	}
 
-	DEBUG(g_print ("_mime_type_from_name for '%s' returned '%s'\n", url, mimetype));
+	DEBUG(g_print ("_get_mime_type_for_name for '%s' returned '%s'\n", url, mimetype));
 	if (mimetype == NULL || strcmp (GNOME_VFS_MIME_TYPE_UNKNOWN, mimetype) == 0) {
 		mimetype = my_gnome_vfs_get_mime_type_with_data (url, &data, parser);
 		DEBUG(g_print ("_get_mime_type_with_data for '%s' returned '%s'\n", url, mimetype ? mimetype : "NULL"));
@@ -2414,7 +2523,13 @@ totem_pl_parser_parse_internal (TotemPlParser *parser, const char *url)
 		return TOTEM_PL_PARSER_RESULT_UNHANDLED;
 	}
 
+	if (strcmp (mimetype, EMPTY_FILE_TYPE) == 0) {
+		g_free (data);
+		return TOTEM_PL_PARSER_RESULT_SUCCESS;
+	}
+
 	if (totem_pl_parser_mimetype_is_ignored (parser, mimetype) != FALSE) {
+		g_free (mimetype);
 		g_free (data);
 		return TOTEM_PL_PARSER_RESULT_IGNORED;
 	}
@@ -2425,6 +2540,11 @@ totem_pl_parser_parse_internal (TotemPlParser *parser, const char *url)
 		for (i = 0; i < G_N_ELEMENTS(special_types); i++) {
 			if (strcmp (special_types[i].mimetype, mimetype) == 0) {
 				DEBUG(g_print ("URL '%s' is special type '%s'\n", url, mimetype));
+				if (parser->priv->disable_unsafe != FALSE && special_types[i].unsafe != FALSE) {
+					g_free (mimetype);
+					g_free (data);
+					return TOTEM_PL_PARSER_RESULT_IGNORED;
+				}
 				ret = (* special_types[i].func) (parser, url, data);
 				found = TRUE;
 				break;
@@ -2435,6 +2555,7 @@ totem_pl_parser_parse_internal (TotemPlParser *parser, const char *url)
 			if (strcmp (dual_types[i].mimetype, mimetype) == 0) {
 				DEBUG(g_print ("URL '%s' is dual type '%s'\n", url, mimetype));
 				if (data == NULL) {
+					g_free (mimetype);
 					mimetype = my_gnome_vfs_get_mime_type_with_data (url, &data, parser);
 				}
 				ret = (* dual_types[i].func) (parser, url, data);
@@ -2448,11 +2569,16 @@ totem_pl_parser_parse_internal (TotemPlParser *parser, const char *url)
 		parser->priv->recurse_level--;
 	}
 
-	if (ret == TOTEM_PL_PARSER_RESULT_SUCCESS)
+	if (ret == TOTEM_PL_PARSER_RESULT_SUCCESS) {
+		g_free (mimetype);
 		return ret;
+	}
 
-	if (totem_pl_parser_ignore_from_mimetype (parser, mimetype))
+	if (totem_pl_parser_ignore_from_mimetype (parser, mimetype)) {
+		g_free (mimetype);
 		return TOTEM_PL_PARSER_RESULT_IGNORED;
+	}
+	g_free (mimetype);
 
 	if (ret != TOTEM_PL_PARSER_RESULT_SUCCESS && parser->priv->fallback) {
 		totem_pl_parser_add_one_url (parser, url, NULL);
