@@ -239,12 +239,10 @@ totem_embedded_open (TotemEmbedded *emb)
 		//disp = gnome_vfs_unescape_string_for_display (totem->mrl);
 		msg = g_strdup_printf(_("Totem could not play '%s'."), disp);
 		g_free (disp);
-		if (err != NULL) {
-			g_message ("error: %s", err->message);
-			totem_interface_error_blocking (msg, err->message,
-				GTK_WINDOW (emb->window));
-		}
 
+		g_message ("error: %s", err->message);
+		totem_interface_error_blocking (msg, err->message,
+				GTK_WINDOW (emb->window));
 		g_free (msg);
 		g_error_free (err);
 
@@ -757,6 +755,7 @@ totem_embedded_push_parser (gpointer data)
 
 	parser = totem_pl_parser_new ();
 	g_object_set (G_OBJECT (parser), "force", TRUE, NULL);
+	g_object_set (G_OBJECT (parser), "disable-unsafe", TRUE, NULL);
 	g_signal_connect (G_OBJECT (parser), "entry", G_CALLBACK (entry_added), emb);
 	res = totem_pl_parser_parse (parser, emb->filename, FALSE);
 	g_object_unref (parser);
@@ -883,7 +882,6 @@ int main (int argc, char **argv)
 	}
 #endif
 	
-	/* FIXME: are we really threaded? */
 	if (XInitThreads () == 0)
 	{
 		gtk_init (&argc, &argv);
@@ -899,7 +897,10 @@ int main (int argc, char **argv)
 		exit (1);
 	}
 
-        // FIXME check that necessary params were given!
+	/* FIXME: are there enough checks? */
+	if (arg_remaining == NULL) {
+		return 1;
+	}
 
 	bacon_video_widget_init_backend (NULL, NULL);
 	gnome_vfs_init ();
@@ -955,8 +956,6 @@ int main (int argc, char **argv)
 
 		emb->window = window;
 	} else {
-		if (emb->filename == NULL)
-			return 1;
 		/* Stand-alone version */
 		emb->window = gtk_window_new (GTK_WINDOW_TOPLEVEL);
 	}
